# File: ComparePerformances.R (19/12/2017)
# Coded by: Mathieu Ribatet
#
# Reference:
# Ben Salah, H., De Gooijer, J.G., Gaunnoun, A., and Ribatet, M. 
#   "Mean-variance and mean-semivariance portfolio selection: A
#    multivariate nonparametric approach".
# Accepted for publication in: Financial Markets and Portfolio Management 
##########################################################################


rm(list=ls())
library(ggplot2)
library(reshape2)
source("OptimalAllocation.R")
source("utils.R")

data <- read.table("donnees.csv", header = TRUE, sep = ";", dec = ",")
date <- as.Date(data[,1], "%d/%m/%y")

## Remove the date column
data <- data[,-1]

## Keep only the asset which have enough data
asset.removed <- c("Credit.Agricole", "Legrand", "Nokia")
data <- data[,!(names(data) %in% asset.removed)]

## Remove the missing values
## Get the corresponding date
f <- function(x)
    any(is.na(x))

idx.na <- apply(data, 1, f)

data <- data[!idx.na,]
date <- date[!idx.na]

## Revert the data so that it starts with the oldest data
data <- data[nrow(data):1,]
date <- rev(date)


## Work with percentages
data <- 100 * data

## Retrieve the CAC 40
CAC40 <- data[,"CAC40"]
data <- data[,which(names(data) != "CAC40")]

## Do a plot of the CAC40 and highlight calm, good and crisis periods
profit <- function(x)
    cumprod(1 + x / 100)

crisis <- as.Date(c("01/01/08", "01/01/09"), "%d/%m/%y")
calm <- as.Date(c("01/01/04", "01/01/05"), "%d/%m/%y")
good <- as.Date(c("01/01/13", "01/01/14"), "%d/%m/%y")

df <- data.frame(Date = date, CAC40 = 6464.12 * profit(CAC40))
df2 <- data.frame(Date = crisis, value = max(df$CAC40))
df3 <- data.frame(Date = calm, value = max(df$CAC40))
df4 <- data.frame(Date = good, value = max(df$CAC40))

plotCAC40 <- ggplot(df, aes(Date)) + ylab("CAC 40") +
    geom_area(data = df2, aes(x = Date, y = value), fill = "red", alpha = 0.5) +
    geom_area(data = df3, aes(x = Date, y = value), fill = "green", alpha = 0.5) +
    geom_area(data = df4, aes(x = Date, y = value), fill = "blue", alpha = 0.5) +
    geom_line(aes(y = CAC40))


p <- list()
all.df <- NULL

positive.weights <- FALSE

for (period in c("Calm", "Crisis", "Good")){
    if (period == "Crisis"){
        train <- date < crisis[1] & date > (crisis[1] - 365)
        test <- date >= crisis[1] & date < crisis[2]
    }

    if (period == "Calm"){
        train <- date < calm[1] & date > (calm[1] - 365)
        test <- date >= calm[1] & date < calm[2]
    }

    if (period == "Good") {
        train <- date < good[1] & date >= (good[1] - 365)
        test <- date >= good[1] & date <= good[2]
    }

    data.train <- data[train,]
    data.vad <- data[test,]

    ## Smooth the data
    c <- 1 ## the scaling factor for the optimal bandwidth
    bandwidth <- c * apply(data.train, 2, bw.SJ)

    univ.mean <- univ.median <- data.train
    for (i in 1:ncol(data.train)){
        univ.mean[,i] <- smoothme(data.train[,i], which = "mean", h = bandwidth[i])
        univ.median[,i] <- smoothme(data.train[,i], which = "median", h = bandwidth[i])
    }

    multi.mean <- smoothme(data.train, which = "mean", h = bandwidth)
    multi.median <- smoothme(data.train, which = "median", h = bandwidth)

    ## Computation of the optimal portoflios
    targetRate <- 0.02
    port.naive <- rep(1, ncol(data.train)) / ncol(data.train)
    port.raw <- dsr(data.train, targetRate = targetRate, positive.weights = positive.weights)$weights
    port.univ.mean <- dsr(univ.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
    port.univ.median <- dsr(univ.median, targetRate = targetRate, positive.weights = positive.weights)$weights
    port.multi.mean <- dsr(multi.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
    port.multi.median <- dsr(multi.median, targetRate = targetRate, positive.weights = positive.weights)$weights

    ## Plot the results, i.e., the value of each of the portfolio on the
    ## validation data set
    rate.naive <- cumprod(1 + as.matrix(data.vad) %*% port.naive / 100)
    rate.raw <- cumprod(1 + as.matrix(data.vad) %*% port.raw / 100)
    rate.univ.mean <- cumprod(1 + as.matrix(data.vad) %*% port.univ.mean / 100)
    rate.univ.median <- cumprod(1 + as.matrix(data.vad) %*% port.univ.median / 100)
    rate.multi.mean <- cumprod(1 + as.matrix(data.vad) %*% port.multi.mean / 100)
    rate.multi.median <- cumprod(1 + as.matrix(data.vad) %*% port.multi.median / 100)
    rate.CAC40 <- cumprod(1 + CAC40[test] / 100)

    df <- data.frame(Date = date[test], CAC40 = rate.CAC40, Naive = rate.naive,
                     Raw = rate.raw, UMean = rate.univ.mean, MMean = rate.multi.mean,
                     UMedian = rate.univ.median, MMedian = rate.multi.median)

    save(df, file = paste("DataFrameForPeriod", period, ".RData", sep = ""))

    df <- melt(df, id = "Date", value.name = "Profit", variable.name = "Method")

    all.df <- rbind(all.df, data.frame(df, Period = period))
    tmp <- ggplot(df, aes(x = Date, y = Profit)) + geom_line(aes(colour = Method)) + scale_x_date(date_labels = "%b %d")

    if (period != "Good")
        tmp <- tmp + theme(legend.position = "none")

    print(tmp)

    p <- c(p, list(tmp))
}

## We will use a trick to get a nice plot
dummy <- all.df
dummy$Date <- as.Date(format(dummy$Date, "%d / %m"), format = "%d / %m")##problem with February 29th 2008 but we don't care
plotResult <- ggplot(dummy, aes(x = Date, y = Profit, color = Period)) + geom_line() +
    facet_grid(. ~ Method) + scale_x_date(date_labels = "%b %d") +
    geom_hline(yintercept = 1, linetype = 2) +
    theme(legend.position = "top", text = element_text(size = 14), axis.text.x = element_text(angle = 45, hjust = 1))

Sys.setlocale("LC_TIME", "en_US")## To get date in English rather than in French


## Showing the plots
print(plotCAC40)
dev.new();
print(plotResult)
