# File: PlotEfficientFrontiers.R (19/12/2017)
# Coded by: Mathieu Ribatet
#
# Reference:
# Ben Salah, H., De Gooijer, J.G., Gaunnoun, A., and Ribatet, M. 
#   "Mean-variance and mean-semivariance portfolio selection: A
#    multivariate nonparametric approach".
# Accepted for publication in: Financial Markets and Portfolio Management 
##########################################################################


rm(list=ls())
library(chron)
library(ggplot2)
library(reshape2)
source("OptimalAllocation.R")
source("utils.R")

data <- read.table("donnees.csv", header = TRUE, sep = ";", dec = ",")
date <- as.Date(data[,1], "%d/%m/%y")

## Remove the date column
data <- data[,-1]

## Keep only the asset which have enough data
asset.removed <- c("Credit.Agricole", "Legrand", "Nokia")
data <- data[,!(names(data) %in% asset.removed)]

## Remove the missing values
## Get the corresponding date
f <- function(x)
    any(is.na(x))

idx.na <- apply(data, 1, f)

data <- data[!idx.na,]
date <- date[!idx.na]

## Revert the data so that it starts with the oldest data
data <- data[nrow(data):1,]
date <- rev(date)


## Work with percentages
data <- 100 * data

## Retrieve the CAC 40
CAC40 <- data[,"CAC40"]
data <- data[,which(names(data) != "CAC40")]

## Do a plot of the CAC40 and highlight calm, good and crisis periods
profit <- function(x)
    cumprod(1 + x / 100)

crisis <- as.Date(c("01/10/07", "31/12/08"), "%d/%m/%y")
calm <- as.Date(c("01/10/03", "31/12/04"), "%d/%m/%y")
good <- as.Date(c("01/10/12", "31/12/13"), "%d/%m/%y")

p <- list()
all.df <- NULL

positive.weights <- FALSE

for (period in c("Calm", "Crisis", "Good")){
    cat("Computations for period", period, "\n")
    if (period == "Crisis")
        idx <- date <= crisis[2] & date >= crisis[1]

    if (period == "Calm")
        idx <- date <= calm[2] & date >= calm[1]

    if (period == "Good")
        idx <- date <= good[2] & date >= good[1]

    sub.data <- data[idx,]
    sub.date <- date[idx]

    df <- data.frame()
    quarters <- quarters(sub.date)
    years <- years(sub.date)
    unique.quarter <- paste("Q", 1:4, sep = "")

    cat("\t Quarter:")
    for (i in c(4,1:3)){##starting from Q4 since then the test period will be Q1
        quarter <- unique.quarter[i]
        cat(quarter, ",")

        train <- which(quarters == quarter & years == max(years))
        data.train <- sub.data[train,]

        if (quarter == "Q4")
            test <- which(quarters == "Q1" & years == max(years))##january of the next year
        else
            test <- which(quarters == unique.quarter[i+1] & years == max(years))

        data.test <- sub.data[test,]
        date.test <- sub.date[test]
        CAC40.test <- CAC40[test]

        ## Smooth the data
        bandwidth <- apply(data.train, 2, bw.SJ)

        univ.mean <- univ.median <- data.train
        for (j in 1:ncol(data.train)){
            univ.mean[,j] <- smoothme(data.train[,j], which = "mean", h = bandwidth[j])$est
            univ.median[,j] <- smoothme(data.train[,j], which = "median", h = bandwidth[j])$est
        }

        multi.mean <- smoothme(data.train, which = "mean", h = bandwidth)$est
        multi.median <- smoothme(data.train, which = "median", h = bandwidth)$est

        ## Computations of the efficient frontiers
        targetRates <- seq(0, 0.1, length = 100)
        risk.naive <- risk.raw <- risk.univ.mean <- risk.univ.median <- risk.multi.mean <-
        risk.multi.median <- risk.adapt.mean <- risk.adapt.median <- rep(NA, length(targetRates))

        for (k in 1:length(targetRates)){
            targetRate <- targetRates[k]

            port.naive <- rep(1, ncol(data.train)) / ncol(data.train)
            port.raw <- dsr(data.train, targetRate = targetRate, positive.weights = positive.weights)$weights
            port.univ.mean <- dsr(univ.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
            port.univ.median <- dsr(univ.median, targetRate = targetRate, positive.weights = positive.weights)$weights
            port.multi.mean <- dsr(multi.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
            port.multi.median <- dsr(multi.median, targetRate = targetRate, positive.weights = positive.weights)$weights
            port.adapt.mean <- adaptivedsr(data.train, targetRate = targetRate, positive.weights = positive.weights, which = "mean")$weights
            port.adapt.median <- adaptivedsr(data.train, targetRate = targetRate, positive.weights = positive.weights, which = "median")$weights

            risk.naive[k] <- sd(as.matrix(data.train) %*% port.naive)
            risk.raw[k] <- sd(as.matrix(data.train) %*% port.raw)
            risk.univ.mean[k] <- sd(as.matrix(data.train) %*% port.univ.mean)
            risk.univ.median[k] <- sd(as.matrix(data.train) %*% port.univ.median)
            risk.multi.mean[k] <- sd(as.matrix(data.train) %*% port.multi.mean)
            risk.multi.median[k] <- sd(as.matrix(data.train) %*% port.multi.median)
            risk.adapt.mean[k] <- sd(as.matrix(data.train) %*% port.adapt.mean)
            risk.adapt.median[k] <- sd(as.matrix(data.train) %*% port.adapt.median)
        }

        df <- data.frame(target.rate = targetRates, Naive = risk.naive,
                         Raw = risk.raw, UMean = risk.univ.mean, MMean = risk.multi.mean,
                         UMedian = risk.univ.median, MMedian = risk.multi.median,
                         AdaptMean = risk.adapt.mean, AdaptMedian = risk.adapt.median)

        save(df, file = paste("Results/FrontierForPeriod", period, "Quarter", quarter, ".RData", sep = ""))

        df <- melt(df, id = "target.rate", value.name="Risk", variable.name = "Method")
        all.df <- rbind(all.df, data.frame(df, Period = period, Quarter = quarter))
    }
}

dummy <- all.df
## We will use a trick to get a nice plot
plotResult <- ggplot(dummy, aes(x = Risk, y = target.rate, color = Method)) + geom_path() +
    facet_grid(Quarter ~ Period) +
    theme(legend.position = "top", text = element_text(size = 14), axis.text.x = element_text(angle = 45, hjust = 1))


## Showing the plots
print(plotResult)

