# File: RollingApproach-year.R (21/12/2017)
# Coded by: Mathieu Ribatet
#
# Reference:
# Ben Salah, H., De Gooijer, J.G., Gaunnoun, A., and Ribatet, M. 
#   "Mean-variance and mean-semivariance portfolio selection: A
#    multivariate nonparametric approach".
# Accepted for publication in: Financial Markets and Portfolio Management 
##########################################################################

library(chron)
library(ggplot2)
library(reshape2)

 multiplot <- function(..., plotlist=NULL, file, cols=1, layout=NULL) {
  library(grid)

  # Make a list from the ... arguments and plotlist
  plots <- c(list(...), plotlist)

  numPlots = length(plots)

  # If layout is NULL, then use 'cols' to determine layout
  if (is.null(layout)) {
    # Make the panel
    # ncol: Number of columns of plots
    # nrow: Number of rows needed, calculated from # of cols
    layout <- matrix(seq(1, cols * ceiling(numPlots/cols)),
                    ncol = cols, nrow = ceiling(numPlots/cols))
  }

 if (numPlots==1) {
    print(plots[[1]])

  } else {
    # Set up the page
    grid.newpage()
    pushViewport(viewport(layout = grid.layout(nrow(layout), ncol(layout))))

    # Make each plot, in the correct location
    for (i in 1:numPlots) {
      # Get the i,j matrix positions of the regions that contain this subplot
      matchidx <- as.data.frame(which(layout == i, arr.ind = TRUE))

      print(plots[[i]], vp = viewport(layout.pos.row = matchidx$row,
                                      layout.pos.col = matchidx$col))
    }
  }
}

dyn.load("Smoothing.dll")## or .dll under Windows
library(MASS)
require(quadprog)

markowitz <- function(data, targetRate = 0.01, positive.weights = FALSE){
    data <- as.matrix(data)

    n.assets <- ncol(data)
    varcov <- cov(data)
    mu <- colMeans(data)

    if (positive.weights) {
        ## consider only positive weights
        d <- rep(0, n.assets)
        A <- rbind(rep(1, n.assets), mu, diag(1, n.assets))
        b <- c(1, targetRate, rep(0, n.assets))
        meq <- 1
        opt <- try(solve.QP(2 * varcov, d, t(A), b, meq = meq), silent = TRUE)

        if (class(opt) == "try-error") {
            weights <- rep(NA, n.assets)
            risk <- NA
        } else {
            weights <- opt$solution
            risk <- var(data %*% weights)
        }
    } else {
        ivarcov <- solve(varcov)
        alpha <- as.numeric(rep(1, n.assets) %*% ivarcov %*% rep(1, n.assets))
        lambda <- as.numeric(t(mu) %*% ivarcov %*% rep(1, n.assets))
        theta <- as.numeric(t(mu) %*% ivarcov %*% mu)

        weights <- (alpha * targetRate - lambda) / (alpha * theta - lambda^2) * ivarcov %*% mu +
            (theta - lambda * targetRate) / (alpha * theta - lambda^2) * ivarcov %*% rep(1, n.assets)
        weights <- as.numeric(weights)

        risk <- (alpha * targetRate^2 - 2 * lambda * targetRate + theta) / (alpha * theta - lambda^2)
    }

    names(weights) <- colnames(data)

    return(list(weights = weights, risk = risk))
}

varPort <- function(data, weights)
    return(var(data %*% weights))

semivar <- function(data, weights, bench = 0, negative = TRUE){
    rp <- data %*% weights
    n.obs <- nrow(data)

    if (negative == TRUE) {
        colSums(pmin(rp - bench, 0)^2) / n.obs
    } else {
        colSums(pmax(rp - bench, 0)^2) / n.obs
    }
}

dsr <- function(data, bench = 0, targetRate = NA, tol = 10^-10, positive.weights = FALSE,
                mu = colMeans(data)){

    data <- as.matrix(data)
    n.assets <- ncol(data)
    n.obs <- nrow(data)
    one <- rep(1, n.assets)

    iMold <- diag(10, n.assets)
    weights.old <- one / n.assets

    weights <- one / n.assets
    ##weights <- markowitz(data, targetRate, positive.weights)$weights

    iter <- 1
    if (positive.weights == TRUE){
        d <- rep(0, n.assets)
        A <- rbind(rep(1, n.assets), mu, diag(1, n.assets))
        b <- c(1, targetRate, rep(0, n.assets))
        meq <- 1

        repeat {
            portReturns <- as.numeric(data %*% weights)
            idx <- which(portReturns < bench)
            M <- matrix(0, n.assets, n.assets)
            for (i in idx)
                M <- M + (data[i,] - bench) %*% t(data[i,] - bench)

            M <- M / n.obs
            iM <- ginv(M)

            opt <- try(solve.QP(2 * M, d, t(A), b, meq = meq), silent = TRUE)

            if (class(opt) == "try-error") {
                weights.old <- weights
                weights <- rep(NA, n.assets)
                risk.semiVar <- NA
                break
            } else {
                weights.old <- weights
                weights <- opt$solution
            }

            delta <- sum(abs(M %*% iMold - diag(n.assets)))
            delta <- sum(abs(weights - weights.old))
            ##print(delta)

            if (delta < tol){
                ## Compute one last time all required quantities to compute the DSR
                portReturns <- as.numeric(data %*% weights)
                idx <- which(portReturns < bench)
                M <- matrix(0, n.assets, n.assets)
                for (i in idx)
                    M <- M + (data[i,] - bench) %*% t(data[i,] - bench)

                M <- M / n.obs

                opt <- try(solve.QP(2 * M, d, t(A), b, meq = meq), silent = TRUE)

                if (class(opt) == "try-error") {
                    weights.old <- weights
                    weights <- rep(NA, n.assets)
                    risk.semiVar <- NA
                    break
                } else {
                    weights.old <- weights
                    weights <- opt$solution
                    risk.semiVar <- semivar(data, weights, bench = bench)
                }

                ## Jump outside of the repeat loop
                break
            }

            iMold <- iM
            iter <- iter + 1
        }
    } else {
        repeat {
            returns <- as.numeric(data %*% weights)

            idx <- which(returns < bench)
            M <- matrix(0, n.assets, n.assets)
            for (i in idx)
                M <- M + (data[i,] - bench) %*% t(data[i,] - bench)

            M <- M / n.obs
            iM <- ginv(M)

            alpha <- as.numeric(one %*% iM %*% one)
            lambda <- as.numeric(t(mu) %*% iM %*% one)
            theta <- as.numeric(t(mu) %*% iM %*% mu)

            if (is.na(targetRate)){
                weights.old <- weights
                weights <- iM %*% one / as.numeric(one %*% iM %*% one)
            } else {
                weights.old <- weights
                weights <- (alpha * targetRate - lambda) / (alpha * theta - lambda^2) *
                    iM %*% mu + (theta - lambda * targetRate) / (alpha * theta - lambda^2) *
                    iM %*% one
            }

            ## The stopping criterion depends on the way (or not!)
            ## with smooth portfolio returns so...
            delta <- sum(abs(M %*% iMold - diag(n.assets)))
            ##delta <- sum(abs(weights - weights.old))
            names(weights) <- NULL


            if ((delta < tol) || (iter > 50)){
                ## Compute one last time all required quantities to compute the DSR
                returns <- as.numeric(data %*% weights)

                idx <- which(returns < bench)
                M <- matrix(0, n.assets, n.assets)
                for (i in idx)
                    M <- M + (data[i,] - bench) %*% t(data[i,] - bench)

                M <- M / n.obs
                iM <- ginv(M)

                alpha <- as.numeric(one %*% iM %*% one)
                lambda <- as.numeric(t(mu) %*% iM %*% one)
                theta <- as.numeric(t(mu) %*% iM %*% mu)

                ## Jump outside of the repeat loop
                break
            }

            weights.old <- weights
            iMold <- iM
            iter <- iter + 1

        }

        risk.semiVar <- (alpha * targetRate^2 - 2 * lambda * targetRate + theta) / (alpha * theta - lambda^2)
    }

    risk.var <- as.numeric(as.numeric(weights) %*% cov(data) %*% as.numeric(weights))

    return(list(weights = weights, risk.semiVar = risk.semiVar, risk.var = risk.var, M = M))
}


smoothme <- function(data, h = apply(data, 2, bw.SJ), return.weights = FALSE, which = "mean"){

    if (is.null(dim(data)))
        dim(data) <- c(length(data), 1)

    n.asset <- ncol(data)
    n.obs <- nrow(data)

    if (which == "mean"){
        tmp <- .C("meanSmoothing", as.integer(n.obs), as.integer(n.asset), as.double(unlist(data)),
                  as.double(h), est = double(n.asset * n.obs), weights = double(n.obs^2))

        smoothing.weights <- matrix(tmp$weights, n.obs)
        smoothing.weights <- smoothing.weights / rowSums(smoothing.weights)

        est <- tmp$est
        dim(est) <- c(n.obs, n.asset)
        est <- data.frame(est)
        names(est) <- names(data)

        if (return.weights)
            return(list(est = est, weights = smoothing.weights))

        else return(list(est=est))
    }

    if (which == "median"){
        est <- matrix(NA, n.obs, n.asset)

        data <- t(data)
        for (i in 1:n.obs){
            fun <- function(z)
                sum(sqrt(colSums((data - z)^2)) * exp(-0.5 * colSums(((data - data[,i]) / h)^2)))

            init <- data[,i]
            est[i,] <- optim(init, fun)$par
        }

        est <- data.frame(est)
        names(est) <- rownames(data)

        return(list(est=est))
    }
}

smoothme2 <- function(data, data2, h = bw.SJ(data2), which = "mean", return.weights = FALSE){
    ## A multivariate smoother for data using the conditional median
    ## but the smoothing weights are obtained from data2 and *not* data

    if (is.null(dim(data)))
        dim(data) <- c(length(data), 1)

    n.obs <- nrow(data)
    n.asset <- ncol(data)

    if (which == "mean"){
        tmp <- .C("meanSmoothing2", as.integer(n.obs), as.integer(n.asset), as.double(unlist(data)),
                  as.double(unlist(data2)), as.double(h), est = double(n.asset * n.obs), weights = double(n.obs^2))

        smoothing.weights <- matrix(tmp$weights, n.obs)
        smoothing.weights <- smoothing.weights / rowSums(smoothing.weights)

        est <- tmp$est
        dim(est) <- c(n.obs, n.asset)
        est <- data.frame(est)
        names(est) <- names(data)

        if (return.weights)
            return(list(est = est, weights = smoothing.weights))

        else return(list(est = est))
    }

    if (which == "median"){
        est <- matrix(NA, n.obs, n.asset)
        data <- t(data)

        for (i in 1:n.obs){
            fun <- function(z)
                sum(sqrt(colSums((data - z)^2)) * exp(-0.5 * n.asset * sum(((data2 - data2[i]) / h)^2)))

            init <- data[,i]
            est[i,] <- optim(init, fun)$par
        }

        est <- data.frame(est)
        names(est) <- rownames(data)

        return(list(est=est))
    }
}

adaptivedsr <- function(data, bench = 0, targetRate = NA, tol = 10^-10, positive.weights = FALSE,
                        h = bw.SJ(raw.port.returns), which = "mean"){

    data <- as.matrix(data)
    n.assets <- ncol(data)
    n.obs <- nrow(data)
    one <- rep(1, n.assets)

    iMold <- diag(10, n.assets)
    weights.old <- one / n.assets

    weights <- one / n.assets

    iter <- 1
    if (positive.weights == TRUE){

        d <- rep(0, n.assets)
        b <- c(1, targetRate, rep(0, n.assets))
        meq <- 1

        repeat {
            raw.port.returns <- as.numeric(data %*% weights)
            smooth.port.returns <- as.matrix(smoothme(raw.port.returns, h, which = which)$est)
            smooth.data <- as.matrix(smoothme2(data, raw.port.returns, h, which = which)$est)

            mu <- colMeans(smooth.data)
            A <- rbind(rep(1, n.assets), mu, diag(1, n.assets))

            idx <- which(smooth.port.returns < bench)
            M <- matrix(0, n.assets, n.assets)
            for (i in idx)
                M <- M + (smooth.data[i,] - bench) %*% t(smooth.data[i,] - bench)

            M <- M / n.obs
            iM <- ginv(M)

            opt <- try(solve.QP(2 * M, d, t(A), b, meq = meq), silent = TRUE)

            if (class(opt) == "try-error") {
                weights.old <- weights
                weights <- rep(NA, n.assets)
                risk.semiVar <- NA
                break
            } else {
                weights.old <- weights
                weights <- opt$solution
            }

            delta <- sum(abs(M %*% iMold - diag(n.assets)))
            delta <- sum(abs(weights - weights.old))
            ##print(delta)

            if (delta < tol){
                ## Compute one last time all required quantities to compute the DSR
                raw.port.returns <- as.numeric(data %*% weights)
                smooth.port.returns <- as.matrix(smoothme(raw.port.returns, h, which = which)$est)
                smooth.data <- as.matrix(smoothme2(data, raw.port.returns, h, which = which)$est)

                mu <- colMeans(smooth.data)
                A <- rbind(rep(1, n.assets), mu, diag(1, n.assets))

                idx <- which(smooth.port.returns < bench)
                M <- matrix(0, n.assets, n.assets)
                for (i in idx)
                    M <- M + (smooth.data[i,] - bench) %*% t(smooth.data[i,] - bench)

                Mnew <- M <- M / n.obs
                iM <- ginv(M)

                opt <- try(solve.QP(2 * M, d, t(A), b, meq = meq), silent = TRUE)

                if (class(opt) == "try-error") {
                    weights <- rep(NA, n.assets)
                    risk.semiVar <- NA
                    break
                } else {
                    weights <- opt$solution
                    risk.semiVar <- semivar(smooth.data, weights, bench = bench)
                }

                ## Jump outside of the repeat loop
                break
            }

            iMold <- iM
            iter <- iter + 1
        }
    } else {
        repeat {
            raw.port.returns <- as.numeric(data %*% weights)
            smooth.port.return <- as.matrix(smoothme(raw.port.returns, h, which = which)$est)
            smooth.data <- as.matrix(smoothme2(data, raw.port.returns, h, which = which)$est)
            mu <- colMeans(smooth.data)

            idx <- which(smooth.port.return < bench)
            M <- matrix(0, n.assets, n.assets)
            for (i in idx)
                M <- M + (smooth.data[i,] - bench) %*% t(smooth.data[i,] - bench)

            M <- M / n.obs
            iM <- ginv(M)

            alpha <- as.numeric(one %*% iM %*% one)
            lambda <- as.numeric(t(mu) %*% iM %*% one)
            theta <- as.numeric(t(mu) %*% iM %*% mu)

            if (is.na(targetRate)){
                weights.old <- weights
                weights <- iM %*% one / as.numeric(one %*% iM %*% one)
            } else {
                weights.old <- weights
                weights <- (alpha * targetRate - lambda) / (alpha * theta - lambda^2) *
                    iM %*% mu + (theta - lambda * targetRate) / (alpha * theta - lambda^2) *
                    iM %*% one
            }

            ## The stopping criterion depends on the way (or not!)
            ## with smooth portfolio returns so...
            delta <- sum(abs(M %*% iMold - diag(n.assets)))
            ##delta <- sum(abs(weights - weights.old))
            names(weights) <- NULL


            if ((delta < tol) || (iter > 50)){
                ## Compute one last time all required quantities to compute the DSR
                raw.port.returns <- as.numeric(data %*% weights)
                smooth.port.return <- as.matrix(smoothme(raw.port.returns, h, which = which)$est)
                smooth.data <- as.matrix(smoothme2(data, raw.port.returns, h, which = which)$est)
                mu <- colMeans(smooth.data)

                idx <- which(smooth.port.return < bench)
                M <- matrix(0, n.assets, n.assets)
                for (i in idx)
                    M <- M + (smooth.data[i,] - bench) %*% t(smooth.data[i,] - bench)

                M <- M / n.obs
                iM <- ginv(M)

                alpha <- as.numeric(one %*% iM %*% one)
                lambda <- as.numeric(t(mu) %*% iM %*% one)
                theta <- as.numeric(t(mu) %*% iM %*% mu)

                ## Jump outside of the repeat loop
                break
            }

            weights.old <- weights
            iMold <- iM
            iter <- iter + 1

        }

        risk.semiVar <- (alpha * targetRate^2 - 2 * lambda * targetRate + theta) / (alpha * theta - lambda^2)
    }

    risk.var <- as.numeric(as.numeric(weights) %*% cov(data) %*% as.numeric(weights))

    return(list(weights = weights, risk.semiVar = risk.semiVar, risk.var = risk.var, M = M))
}

#################### MAIN  #####################
data <- read.table("c:/Gannnoun-2017/Modified-Paper-August22-2017/Computer-codes-Mathieu/Jan/donnees.csv", header = TRUE, sep = ";",dec = ",")
date <- as.Date(data[,1], "%d/%m/%y")

## Remove the date column
data <- data[,-1]

## Keep only the asset which have enough data
asset.removed <- c("Credit.Agricole", "Legrand", "Nokia")
data <- data[,!(names(data) %in% asset.removed)]

## Remove the missing values
## Get the corresponding date
f <- function(x)
    any(is.na(x))

idx.na <- apply(data, 1, f)

data <- data[!idx.na,]
date <- date[!idx.na]

## Revert the data so that it starts with the oldest data
data <- data[nrow(data):1,]
date <- rev(date)


## Work with percentages
data <- 100 * data

## Retrieve the CAC 40
CAC40 <- data[,"CAC40"]
data <- data[,which(names(data) != "CAC40")]

## Do a plot of the CAC40 and highlight calm, good and crisis periods
profit <- function(x)
    cumprod(1 + x / 100)

crisis <- as.Date(c("01/10/07", "31/12/08"), "%d/%m/%y")
calm <- as.Date(c("01/10/03", "31/12/04"), "%d/%m/%y")
good <- as.Date(c("01/10/12", "31/12/13"), "%d/%m/%y")

p <- list()
all.df <- NULL

positive.weights <- FALSE

for (period in c("Calm", "Crisis", "Good")){
#    cat("Computations for period", period, "\n")
    if (period == "Crisis")
        idx <- date <= crisis[2] & date >= (crisis[1] - 365)

    if (period == "Calm"){
#        idx <- date <= calm[2] & date >= (calm[1] - 365)
         train <- date <= calm[2] & date >= (calm[1] - 365)
         test <- date >= calm[1] & date < calm[2]
    }        

    if (period == "Good")
        idx <- date <= good[2] & date >= (good[1] - 365)

#    sub.data <- data[idx,]
#    sub.date <- date[idx]

    data.train <- data[train,]
    data.vad <- data[test,]

    df <- data.frame()
#    quarters <- quarters(sub.date)
#    years <- years(sub.date)
#    unique.quarter <- paste("Q", 1:4, sep = "")

#    cat("\t Quarter:")
#    for (i in c(4,1:3)){##starting from Q4 since then the test period will be Q1
#        quarter <- unique.quarter[i]
#        cat(quarter, ",")

#        train <- which(quarters == quarter & years == max(years))
#        data.train <- sub.data[train,]

#        if (quarter == "Q4")
#            test <- which(quarters == "Q1" & years == max(years))##january of the next year
#        else
#            test <- which(quarters == unique.quarter[i+1] & years == max(years))

#        data.test <- sub.data[test,]
#        date.test <- sub.date[test]
#        CAC40.test <- CAC40[test]

        ## Smooth the data
        bandwidth <- apply(data.train, 2, bw.SJ)

        univ.mean <- univ.median <- data.train
        for (j in 1:ncol(data.train)){
            univ.mean[,j] <- smoothme(data.train[,j], which = "mean", h = bandwidth[j])$est
            univ.median[,j] <- smoothme(data.train[,j], which = "median", h = bandwidth[j])$est
        }

        multi.mean <- smoothme(data.train, which = "mean", h = bandwidth)$est
        multi.median <- smoothme(data.train, which = "median", h = bandwidth)$est

        ## Computation of the optimal portoflio
        targetRate <- 0.01
        port.naive <- rep(1, ncol(data.train)) / ncol(data.train)
        port.raw <- dsr(data.train, targetRate = targetRate, positive.weights = positive.weights)$weights
        port.univ.mean <- dsr(univ.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
        port.univ.median <- dsr(univ.median, targetRate = targetRate, positive.weights = positive.weights)$weights
        port.multi.mean <- dsr(multi.mean, targetRate = targetRate, positive.weights = positive.weights)$weights
        port.multi.median <- dsr(multi.median, targetRate = targetRate, positive.weights = positive.weights)$weights
        port.adapt.mean <- adaptivedsr(data.train, targetRate = targetRate, positive.weights = positive.weights, which = "mean")$weights
        port.adapt.median <- adaptivedsr(data.train, targetRate = targetRate, positive.weights = positive.weights, which = "median")$weights

        ## Plot the results, i.e., the value of each of the portfolio on the
        ## validation (=test) data set
        rate.naive <- profit(as.matrix(data.vad) %*% port.naive)
        rate.raw <- profit(as.matrix(data.vad) %*% port.raw)
        rate.univ.mean <- profit(as.matrix(data.vad) %*% port.univ.mean)
        rate.univ.median <- profit(as.matrix(data.vad) %*% port.univ.median)
        rate.multi.mean <- profit(as.matrix(data.vad) %*% port.multi.mean)
        rate.multi.median <- profit(as.matrix(data.vad) %*% port.multi.median)
        rate.adapt.mean <- profit(as.matrix(data.vad) %*% port.adapt.mean)
        rate.adapt.median <- profit(as.matrix(data.vad) %*% port.adapt.median)
        rate.CAC40 <- profit(CAC40[test])

##        df <- rbind(df,
         df <-      data.frame(Date = date[test], CAC40 = rate.CAC40, Naive = rate.naive,
                               Raw = rate.raw, UMean = rate.univ.mean, MMean = rate.multi.mean,
                               UMedian = rate.univ.median, MMedian = rate.multi.median,
                               AdaptMean = rate.adapt.mean, AdaptMedian = rate.adapt.median)
##  }

    cat("\n")

#    save(df, file = paste("Results/DataFrameForPeriod", period, "PositiveWeights.RData", sep = ""))

    df <- melt(df, id = "Date", value.name = "Profit", variable.name = "Method")

    all.df <- rbind(all.df, data.frame(df, Period = period))
    tmp <- ggplot(df, aes(x = Date, y = Profit)) + geom_line(aes(colour = Method)) + scale_x_date(date_labels = "%b %d")

    if (period != "Good")
        tmp <- tmp + theme(legend.position = "none")

    ##print(tmp)

    p <- c(p, list(tmp))
}

## We will use a trick to get a nice plot
dummy <- all.df
dummy$Date <- as.Date(format(dummy$Date, "%d / %m"), format = "%d / %m")##problem with February 29th 2008 but we don't care
plotResult <- ggplot(dummy, aes(x = Date, y = Profit, color = Method)) + geom_line() +
    facet_grid(. ~ Period) + scale_x_date(date_labels = "%b %d") +
    geom_hline(yintercept = 1, linetype = 2) +
    theme(legend.position = "top", text = element_text(size = 14), axis.text.x = element_text(angle = 45, hjust = 1))

# Sys.setlocale("LC_TIME", "en_US")## To get date in English rather than in French


## Showing the plots
print(plotResult)


